<?php

/**
 * @file
 * Contains \Drupal\direct_queue\Command\RunCommand.
 */

namespace Drupal\direct_queue\Commands;

use Drupal\Console\Core\Command\Command;
use Drupal\Core\Queue\SuspendQueueException;
use Drush\Commands\DrushCommands;
use Error;

/**
 * Class RunCommand.
 *
 * @package Drupal\direct_queue
 */
class RunCommand extends DrushCommands {
  /**
   * Execute job.
   *
   * @command direct_queue:run
   * @param $item_id Which item do you want to execute?
   * @param $expire The expire to match against to make sure we dont fetch the same item twice.
   * @option uppercase Uppercase the text
   */
  public function run($item_id, $expire, $options = ['uppercase' => FALSE]) {
    // Get some basic services.
    $connection = \Drupal::getContainer()->get('database');
    $queue_manager = \Drupal::getContainer()->get('plugin.manager.queue_worker');

    // Try to fetch our item from the queue.
    // Make sure the expire value matches ours.
    $item = $connection->queryRange('SELECT data, created, item_id, name FROM {queue} q WHERE item_id = :item_id AND expire = :expire ORDER BY created, item_id ASC', 0, 1, array(':item_id' => preg_replace('/[^0-9]/', '', $item_id), ':expire' => preg_replace('/[^0-9]/', '', $expire)))->fetchObject();

    // Check if we were able to fetch our item.
    if (!$item) {
      $e = new \Exception("Could not load queue item: ".preg_replace('/[^0-9]/', '', $item_id). " expiration: ".preg_replace('/[^0-9]/', '', $expire));
      \Drupal::logger('direct_queue')->error($e);
      $this->output()->writeln($e->getMessage());
      return;
    }

    // Unserialize the data from the database.
    $item->data = unserialize($item->data);

    // Get the queue for this queue item.
    $queue = \Drupal::queue($item->name);

    try {

      // Try to start a queue worker and process the queue item.
      $queue_worker = $queue_manager->createInstance($item->name);
      $queue_worker->processItem($item->data);

      // Remove the item from the queue.
      $queue->deleteItem($item);

      // Tell the backend we had success.
      $this->output()->writeln('Success');
    }
    catch (SuspendQueueException $e) {
      // If the worker indicates there is a problem with the whole queue,
      // release the item and skip to the next queue.
      $queue->releaseItem($item);
      \Drupal::logger('direct_queue')->error($e);
      $this->output->writeln($e->getMessage());
    }
    catch (\Exception $e) {
      // In case of any other kind of exception, log it and leave the item
      // in the queue to be processed again later.
      // Drupal Cron will clean up when expire < time().
      \Drupal::logger('direct_queue')->error($e);
      $this->output->writeln($e->getMessage());
    }
  }
}
